//
//  DigitalOutputController.m
//  DigitalOutput
//
//  Created by Phidgets on 2015-07-14.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "DigitalOutputController.h"

@implementation DigitalOutputController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetDigitalOutput_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetDigitalOutput_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks

static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, and error events
- (void)onAttachHandler{
    PhidgetReturnCode result;
    NSArray *supportedForwardVoltages;
    Phidget_DeviceID deviceID;
    Phidget_ChannelSubclass channelSubclass;
    double maxCurrentLimit, minCurrentLimit, currentLimit, minDutyCycle, maxDutyCycle, dutyCycle;
    PhidgetDigitalOutput_LEDForwardVoltage ledForwardVoltage;
    int outputState;
    
    //Get information from channel which will allow us to configure the GUI properly
    result = Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = Phidget_getChannelSubclass((PhidgetHandle)ch, &channelSubclass);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetDigitalOutput_getState(ch, &outputState);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    [settingsBoxLED setHidden:YES];
    
    switch(deviceID)
    {
		case PHIDID_1031:
		case PHIDID_1032:
            supportedForwardVoltages = [[NSArray alloc] initWithObjects:@"1.7 V", @"2.75 V", @"3.9 V", @"5.0 V", nil];
            [forwardVoltageCombo addItemsWithTitles: supportedForwardVoltages];
            break;
        case PHIDID_LED1000:
            supportedForwardVoltages = [[NSArray alloc] initWithObjects:@"3.2 V",@"4.0 V", @"4.8 V", @"5.6 V", nil];
            [forwardVoltageCombo addItemsWithTitles: supportedForwardVoltages];
            break;
        default:
            break;
    }
    
    if(channelSubclass == PHIDCHSUBCLASS_DIGITALOUTPUT_LED_DRIVER){
        [settingsBoxLED setHidden:NO];
        //get more information about device
        result = PhidgetDigitalOutput_getMaxLEDCurrentLimit(ch, &maxCurrentLimit);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetDigitalOutput_getMinLEDCurrentLimit(ch, &minCurrentLimit);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetDigitalOutput_getLEDCurrentLimit(ch, &currentLimit);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetDigitalOutput_getLEDForwardVoltage(ch, &ledForwardVoltage);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [currentLimitSlider setMaxValue:maxCurrentLimit];
        [currentLimitSlider setMinValue:minCurrentLimit];
        [currentLimitMaxLabel setStringValue:[NSString stringWithFormat:@"%.3f", maxCurrentLimit]];
        [currentLimitMinLabel setStringValue:[NSString stringWithFormat:@"%.3f", minCurrentLimit]];
        [currentLimitLabel setStringValue:[NSString stringWithFormat:@"%.3f A", currentLimit]];
        [currentLimitSlider setDoubleValue:currentLimit];
        [forwardVoltageCombo selectItemWithTitle: (ledForwardVoltage == LED_FORWARD_VOLTAGE_1_7V) ? @"1.7 V" : (ledForwardVoltage == LED_FORWARD_VOLTAGE_2_75V) ? @"2.75 V" : (ledForwardVoltage == LED_FORWARD_VOLTAGE_3_2V) ? @"3.2 V" : (ledForwardVoltage == LED_FORWARD_VOLTAGE_3_9V) ? @"3.9 V" : (ledForwardVoltage == LED_FORWARD_VOLTAGE_4_0V) ? @"4.0 V" : (ledForwardVoltage == LED_FORWARD_VOLTAGE_4_8V) ? @"4.8 V" : (ledForwardVoltage == LED_FORWARD_VOLTAGE_5_0V) ? @"5.0 V" : @"5.6 V"];
    }
    
    if(channelSubclass == PHIDCHSUBCLASS_DIGITALOUTPUT_DUTY_CYCLE 
	  || channelSubclass == PHIDCHSUBCLASS_DIGITALOUTPUT_FREQUENCY 
	  || channelSubclass == PHIDCHSUBCLASS_DIGITALOUTPUT_LED_DRIVER){
        result = PhidgetDigitalOutput_getDutyCycle(ch, &dutyCycle);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetDigitalOutput_getMinDutyCycle(ch, &minDutyCycle);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [dutyCycleBox setHidden:NO];
        NSRect rect = [stateButton frame];
        rect.origin.x = dutyCycleLabel.frame.origin.x + 50;
        [stateButton setFrame:rect];
        PhidgetDigitalOutput_getMaxDutyCycle(ch, &maxDutyCycle);
        [dutyCycleSlider setMinValue:minDutyCycle];
        [dutyCycleSlider setMaxValue:maxDutyCycle];
        [dutyCycleSlider setDoubleValue:dutyCycle];
        [minDutyCycleLabel setStringValue:[NSString stringWithFormat:@"%.2f",minDutyCycle]];
        [maxDutyCycleLabel setStringValue:[NSString stringWithFormat:@"%.2f",maxDutyCycle]];
        [dutyCycleLabel setStringValue:[NSString stringWithFormat:@"%.2f",dutyCycle]];
    }
    else{
        [dutyCycleBox setHidden:YES];
        NSRect rect = [stateButton frame];
        rect.origin.x = (settingsBoxLED.frame.size.width / 2) - rect.size.width / 2;
        [stateButton setFrame:rect];
    }
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    if(settingsBoxLED.isHidden)
        frame.size.height = [mainWindow maxSize].height - settingsBoxLED.frame.size.height;
    else
        frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [forwardVoltageCombo removeAllItems];
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)send{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

#pragma mark GUI Controls
- (IBAction)setDutyCycle:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetDigitalOutput_setDutyCycle(ch, [dutyCycleSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [dutyCycleLabel setStringValue:[NSString stringWithFormat:@"%0.2f",[dutyCycleSlider doubleValue]]];
}
- (IBAction)setCurrentLimit:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetDigitalOutput_setLEDCurrentLimit(ch, ([currentLimitSlider doubleValue]));
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [currentLimitLabel setStringValue:[NSString stringWithFormat:@"%0.3f A",[currentLimitSlider doubleValue]]];
}
- (IBAction)setOnOff:(id)sende{
    PhidgetReturnCode result;
    int state;
    result = PhidgetDigitalOutput_getState(ch, &state);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    result = PhidgetDigitalOutput_setState(ch, (state == 0) ? 1 : 0);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    if(state){
        [outputStateButton setTitle:@"ON"];
        [dutyCycleLabel setStringValue:@"0.00"];
        [dutyCycleSlider setDoubleValue:[dutyCycleSlider minValue]];
    }
    else{
        [outputStateButton setTitle:@"OFF"];
        [dutyCycleLabel setStringValue:@"1.00"];
        [dutyCycleSlider setDoubleValue:[dutyCycleSlider maxValue]];
    }
    
}
- (IBAction)setForwardVoltage:(id)sender{
    PhidgetReturnCode result = EPHIDGET_OK;
    if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"1.7 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_1_7V);
    else if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"2.75 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_2_75V);
    else if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"3.2 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_3_2V);
    else if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"3.9 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_3_9V);
    else if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"4.0 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_4_0V);
    else if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"4.8 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_4_8V);
    else if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"5.0 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_5_0V);
    else if([forwardVoltageCombo.titleOfSelectedItem  isEqual: @"5.6 V"])
        result = PhidgetDigitalOutput_setLEDForwardVoltage(ch, LED_FORWARD_VOLTAGE_5_6V);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}


@end
